local ADDON_NAME, HonorTracker = ...
local BracketsSync = HonorTracker:RegisterModule("BracketsSync")
local AceSerializer = LibStub("AceSerializer-3.0")
local Comms = HonorTracker:GetModule("Comms")
local PLAYER_NAME = UnitName("player")
local MAX_BOOTSTRAP_TIME = 3600 * 6
local ADDON_VERSION = GetAddOnMetadata(ADDON_NAME, "Version")

local throttles = {}

-- Someone is requesting our data
function BracketsSync:Ping(requester)
    local currentTime = GetServerTime()
    local totalPlayers = 0
    for name, data in pairs(self.realmBracketDB.players) do
        totalPlayers = totalPlayers + 1
    end

    Comms:SendPrivateMessage(requester, "pong", {totalPlayers = totalPlayers, version = ADDON_VERSION})
end

-- Bootstrap another players data set
function BracketsSync:BootstrapData(requester)
    -- Only allow people to request a bootstrap every 6 hours
    local currentTime = GetServerTime()
    if( throttles[requester] and (GetServerTime() - throttles[requester]) <= MAX_BOOTSTRAP_TIME) then
        return
    end

    -- Find all players we have semi-recent data for
    for name, data in pairs(self.realmBracketDB.players) do
        Comms:SendPrivatePlayerData(requester, name, data)
    end
end

-- Convert HonorSpy into our format
function BracketsSync:SaveHonorSpy(sender, playerName, data)
    if( type(playerName) ~= "string" ) then return end
    if( playerName == PLAYER_NAME ) then return end

    if( string.find(playerName, "[%d%p%s%c%z]") ) then return end
    if( type(data.last_checked) ~= "number" or data.last_checked > GetServerTime() ) then return end
    if( type(data.RP) ~= "number" or type(data.rankProgress) ~= "number" or type(data.rank) ~= "number" ) then return end
    if( type(data.class) ~= "string" or not RAID_CLASS_COLORS[data.class] ) then return end
    if( type(data.standing) ~= "number" or type(data.lastWeekHonor) ~= "number" or type(data.thisWeekHonor) ~= "number" ) then return end
    if( data.thisWeekHonor <= 0 ) then return end

    -- Filter out data prior to the reset
    if( self.realmBracketDB.resetAt and data.last_checked < self.realmBracketDB.resetAt) then return end

    local metadata = self.realmBracketDB.playersMeta[playerName]
    -- If we've seen the person today then our data takes priority
    if( metadata and metadata.seenToday ) then return end

    -- Filter out data when ours is newer
    local existingData = self.realmBracketDB.players[playerName]
    if( existingData and existingData.lastChecked > data.last_checked ) then return end
    
    self:Debug("Syncing %s from %s (via HonorSpy)", playerName, sender)
    
    self.realmBracketDB.playersMeta[playerName] = self.realmBracketDB.playersMeta[playerName] or {}
    self.realmBracketDB.playersMeta[playerName].sender = sender

    self.realmBracketDB.players[playerName] = {
        class = data.class,
        lastChecked = data.last_checked,
        rankPoints = data.RP,
        rankProgress = data.rankProgress,
        rank = data.rank,
        thisWeek = {
            honor = data.thisWeekHonor,
            kills = 15, -- HonorSpy only gives us an assumption that they hit 15
        },
        lastWeek = {
            standing = data.standing,
            honor = data.lastWeekHonor,
        }
    }
end

-- Save our own data format
function BracketsSync:SaveData(sender, playerName, data)
    if( type(playerName) ~= "string" ) then return end
    if( string.find(playerName, "[%d%p%s%c%z]") ) then return end
    if( playerName == PLAYER_NAME ) then return end

    local metadata = self.realmBracketDB.playersMeta[playerName]
    -- If we've seen the person today then our data takes priority
    if( metadata and metadata.seenToday ) then return end

    data = Comms:DeserializePlayerData(data)
    if( not data ) then return end

    -- Filter out data when ours is newer
    local existingData = self.realmBracketDB.players[playerName]
    if( existingData and existingData.lastChecked > data.last_checked ) then return end
    
    self:Debug("Syncing %s from %s (via HonorTracker)", playerName, sender)

    self.realmBracketDB.playersMeta[playerName] = self.realmBracketDB.playersMeta[playerName] or {}
    self.realmBracketDB.playersMeta[playerName].sender = sender

    self.realmBracketDB.players[playerName] = {
        class = data.class,
        lastChecked = data.lastChecked,
        rankPoints = data.rankPoints,
        rankProgress = data.rankProgress,
        rank = data.rank,
        thisWeek = {
            honor = data.thisWeek.honor,
            kills = data.thisWeek.kills,
        },
        lastWeek = {
            standing = data.lastWeek.standing,
            honor = data.lastWeek.honor,
        }
    }
end

function BracketsSync:OnMessage(sender, type, ...)
    -- Resync against HonorSpy format
    if( type == "HonorSpy" ) then
        local playerName, data = ...
        if( playerName == "filtered_players" ) then
            for playerName, player in pairs(data) do
                self:SaveHonorSpy(sender, playerName, player)
            end
        else
            self:SaveHonorSpy(sender, playerName, data)
        end
    elseif( type == "push" ) then
        self:SaveData(sender, unpack(...))
    elseif( type == "bootstrap" ) then
        self:BootstrapData(sender)
    elseif( type == "ping" ) then
        self:Ping(sender)
    end
end
